#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/select.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "main.h"
#include "filter.h"
#include "handlers.h"

struct FilterRules *client_fr = 0;

static void loop(Net_c_prepare_read_fdset net_prepare,
        Net_c_process_read_fdset net_process,
        Net_c_send net_send)
{
    fd_set read_set;
    int maxfd;
    int res;

    do
    {
        FD_ZERO(&read_set);

        FD_SET(0, &read_set); /* stdin */
        maxfd = 0;

        net_prepare(&read_set, &maxfd);

        res = select(maxfd + 1, &read_set, 0, 0, 0);
        if (res == -1)
        {
            if (errno == EINTR)
                continue;
            else
                error("Error in select()");
        }

        res = net_process(&read_set);
        if (res == -1) /* EOF */
            break;

        if (FD_ISSET(0, &read_set))
        {
            res = read(0, stream_buffer, stream_buffer_size);
            if (res == 0) /* EOF */
                break;
            net_send(stream_buffer, res);
        }
    } while (1);
}

int client()
{
    Net_c_prepare_read_fdset net_prepare_read_fdset;
    Net_c_process_read_fdset net_process_read_fdset;
    Net_c_send net_send;

    /* Prepare the filter */
    {
        struct FFilter *ff;
        client_fr = new_filter_rules();
        ff = new_ftelnet();
        add_ffilter(client_fr, ff);
    }

    if (command_line.c_param.transport == UNIX)
        /* Will be 'tcp', 'ether', ... */
    {
        c_unix_connect_socket();
        net_prepare_read_fdset = c_unix_prepare_read_fdset;
        net_process_read_fdset = c_unix_process_read_fdset;
        net_send = c_unix_send;
    }

    if (command_line.c_param.raw_mode)
        prepare_user_terminal();

    loop(net_prepare_read_fdset, net_process_read_fdset, net_send);

    if (command_line.c_param.raw_mode)
        restore_user_terminal();

    return 0;
}
